#include <iostream>
#include <vector>

// La primera solucin que se puede plantear sera la siguiente.
//template <typename Contenedor>
//std::ostream &operator<<(std::ostream &out, const Contenedor &c)
//{
//    for (typename Contenedor::const_iterator i = c.begin(); i != c.end();) {
//        out << *i;
//        if (++i != c.end())
//            out << ' ';
//    }
//    return out;
//}

// Sin embargo el cdigo anterior no compila correctamente.
// correctamente. El problema radica en que la sobrecarga del operator<< para
// los tipos primitivos se vuelve ambigua, ya que el operador no se define
// para std::ostream si no para un tipo std::basic_ostream de plantilla, lo que
// povoca que la expresin out << ' ' encaje en al menos 2 sobrecargas por 
// medio de la deduccin de parmetros. La solucin es no declarar la 
// sobrecarga con std::ostream si no con el tipo bsico que usan todas las 
// sobrecargas predefinidas, con lo que quedara un cdigo as:
template <typename Contenedor, typename _CharT, typename _Traits>
std::basic_ostream<_CharT, _Traits>&
operator<<(std::basic_ostream<_CharT, _Traits> &out, const Contenedor &c)
{
    for (typename Contenedor::const_iterator i = c.begin(); i != c.end();) {
        out << *i;
        if (++i != c.end())
            out << ' ';
    }
    return out;
}

// En versiones anteriores de g++ y la stdlibc++, podemos declarar una
// sobrecarga ms specializada para evitar el problema anterior de una
// manera ms sencilla.
template <typename T>
std::ostream &operator<<(std::ostream &out, const std::vector<T> &v)
{
    for (typename std::vector<T>::const_iterator i = v.begin(); 
         i != v.end();) {
        out << *i;
        if (++i != v.end())
            out << ' ';
    }
    return out;
}

// Prueba de la sobrecarga.
int main(int argc, char *argv[])
{
    int array[] = {1, 2, 3, 4};
    std::vector<int> v(array, array + 4);

    std::cout << v << std::endl;
}

